# -*- coding: iso-8859-15 -*-
"""constants.py

Implements all constants which are required for PyAuthD. Calling this file as
__main__ will automatically generate an appropriate C-header file which needs
to be moved to ../include.

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""

# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 147 $"
__date__ = "$LastChangedDate: 2005-11-04 09:51:01 +0100 (pią, 04 lis 2005) $"

# Special constants for servers.
from config import SOCK_PATH, RUN_PATH

# Global constants for servers.
SERVER_QUIT = 0
PAM = 1
PAM_CLEANUP = 2
NSS = 3
PPP = 4
POSTFIX = 5
SQUIRRELMAIL = 6

# PAM-related constants.
PAM_RETURN = 0
PAM_SET_DATA = 1
PAM_GET_DATA = 2
PAM_SET_ITEM = 3
PAM_GET_ITEM = 4
PAM_GET_USER = 5
PAM_PUTENV = 6
PAM_GETENV = 7
PAM_GETENVLIST = 8
PAM_GET_ERROR = 9
PAM_CALL_CONV = 10
PAM_FAIL_DELAY = 11

# NSS-related constants.
NSS_GETPWUID = 1
NSS_GETPWNAM = 2
NSS_GETPWENT = 3
NSS_GETSPUID = 4
NSS_GETSPNAM = 5
NSS_GETSPENT = 6
NSS_GETGRGID = 7
NSS_GETGRNAM = 8
NSS_GETGRENT = 9

# PPP-related constants.
PPP_GET_SECRET = 1
PPP_CHOOSE_IP = 2
PPP_CHECK_IP = 3
PPP_CHECK_IDLE = 4
PPP_CHECK_HOLDOFF = 5
PPP_SIGNAL_PIDCHANGE = 6
PPP_SIGNAL_PHASECHANGE = 7
PPP_SIGNAL_SIGRECEIVED = 8
PPP_SIGNAL_IPUP = 9
PPP_SIGNAL_IPDOWN = 10
PPP_SIGNAL_AUTHUP = 11
PPP_SIGNAL_LINKDOWN = 12
PPP_SIGNAL_EXIT = 13
PPP_CLOSE_ERROR = 14
PPP_CLOSE_FINISH = 15

# Postfix-related constants.
POSTFIX_LOOKUP = 1
POSTFIX_UPDATE = 2
POSTFIX_DELETE = 3
POSTFIX_SEQUENCE = 4
POSTFIX_CLOSE = 5

# Squirrelmail related constants.
SQUIRRELMAIL_GETNAMEALIAS = 1
SQUIRRELMAIL_SETALIAS = 2
SQUIRRELMAIL_CLOSE = 3

def _declareConstants():
    outfile = file("PyAuthD_constants.h","w")

    # Write header.
    outfile.write("""
/* PyAuthD_constants.h
   -------------------

   Autogenerated declarations for all bindings from constants.py.

   Copyright (C) 2005, Heiko Wundram.
   Released under a BSD-style license, see LICENSE for details. */

#ifndef __INCLUDE_PYAUTHD_CONSTANTS_H
#define __INCLUDE_PYAUTHD_CONSTANTS_H

/* Constant declarations. */

"""[1:])

    # Write out all globals which are integers.
    for n, v in globals().iteritems():
        if not n.startswith("_"):
            if isinstance(v,int):
                outfile.write("#define AUTHD_%s %i\n" % (n,v))
            elif isinstance(v,str):
                outfile.write("#define AUTHD_%s \"%s\"\n" % (n,v))
        elif n == "__author__":
            outfile.write("#define CONSTANTS_AUTHOR \"%s\"\n" % (v,))
        elif n == "__version__":
            outfile.write("#define CONSTANTS_REVISION \"%s\"\n" % (v,))
        elif n == "__date__":
            outfile.write("#define CONSTANTS_DATE \"%s\"\n" % (v,))

    # Write footer.
    outfile.write("""
#endif /* __INCLUDE_PYAUTHD_CONSTANTS_H */
""")

    # Close file.
    outfile.close()

# Create file with constants if called.
if __name__ == "__main__":
    _declareConstants()
