# -*- coding: iso-8859-15 -*-
"""daemonize.py

Implements daemonization. This works by implementing a daemonize method which
does all steps necessary to background the current process.

It's based on the ASPN python daemon entry from Chad J. Schroeder.

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""
# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 79 $"
__date__ = "$LastChangedDate: 2005-10-24 03:00:23 +0200 (pon, 24 paź 2005) $"

# Python imports.

import atexit
import fcntl
import os
import resource
import signal
import sys

# Our imports.

import config

# Methods that check whether the run file is okay.

def acquireRunFile():
    # Try to open existing run file.
    try:
        f = file(config.RUN_FILE,"r+")
    except:
        # No existing run file, try to open new file by truncating. This
        # might, in case two concurrent processes are started at the same time,
        # mean that the runfile contains no PID. This is irrelevant, anyway.
        try:
            f = file(config.RUN_FILE,"w+")
        except IOError:
            raise RuntimeError, "Cannot open lockfile, is dir writable?"

    # Lock runfile to exclusive access. This means that if another process
    # tries to lock it, it will fail.
    try:
        fcntl.flock(f.fileno(),fcntl.LOCK_EX|fcntl.LOCK_NB)
    except IOError:
        raise RuntimeError, "Cannot lock run file exclusively, running?"

    # Truncate lock file, it's written to by daemonize.
    f.truncate()

    # Save lock file in config to have it when closing.
    config.RUN_FILE = f
    atexit.register(releaseRunFile)

def releaseRunFile():
    # Close run file after it has been opened and locked.
    fcntl.flock(config.RUN_FILE.fileno(),fcntl.LOCK_UN)
    config.RUN_FILE.close()

def daemonize():
    # Fork.
    try:
        pid = os.fork()
    except OSError:
        raise RuntimeError, "Cannot do first fork."

    # Are we in the first child?
    if not pid:
        # Acquire run file.
        acquireRunFile()

        # New process leader and ignore signal SIGHUP.
        os.setsid()
        signal.signal(signal.SIGHUP, signal.SIG_IGN)

        # Fork again.
        try:
            pid = os.fork()
        except OSError:
            raise RuntimeError, "Cannot do second fork."

        # Are we in the second child?
        if not pid:
            # Write out pid to runfile.
            config.RUN_FILE.write("%s\n" % (os.getpid(),))
            config.RUN_FILE.flush()

            # Change to vmail directory.
            os.chdir(config.VMAIL_HOME)
            # os.umask(), set umask, TODO
        else:
            # We are the first child, just quit and make sure that atexit isn't
            # called.
            os._exit(0)
    else:
        # We are the parent, just quit and make sure that atexit isn't called.
        os._exit(0)

    # Close the terminal descriptors, leave all others open!
    for i in range(3):
        try:
            os.close(i)
        except OSError:
            pass

    # Redirect std-in, -out and -err to null device.
    os.open("/dev/null",os.O_RDWR)
    os.dup2(0,1)
    os.dup2(0,2)
