# -*- coding: iso-8859-15 -*-
"""pyauthd_postfix.py

Implementation of postfix hash handling through this module. The module is
called if in the postfix configuration you specify PyAuthD:<name>.

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""

# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 119 $"
__date__ = "$LastChangedDate: 2005-10-29 07:32:54 +0200 (sob, 29 paź 2005) $"

# Python imports.

import errno
import time

# Our imports.

from pyauthd_serv import BaseMixin
from constants import *
from Postfix_constants import *

# NSS specific stuff.

class PostfixMixin(BaseMixin):
    """Mixin class for BasePyAuthDClient which handles incoming Postfix
    communication."""

    # Main handling method. This is called by the Clients handle() method.

    def handle_POSTFIX(self):
        # Receive opening data.
        name = self.recvData()
        open_flags = self.recvValue()
        dict_flags = self.recvValue()

        # TODO: Initialize dict based on values.
        self.sendValue(self.handleInit(name,open_flags,dict_flags))
        self.sendValue(int(time.time()))

        # Loop while there are still commands left.
        while True:
            # Initialize communication for single command.
            rv = None
            comm = self.recvValue()

            # Switch on commands.
            try:
                if comm == POSTFIX_LOOKUP:
                    rv = self.handleLookup(self.recvData())
                elif comm == POSTFIX_CLOSE:
                    break

                # We got some form of answer, don't do anything for now.
                self.sendValue(ERR_NONE)
                self.sendData(rv)
            except:
                # Some exception occured, send error code and stop.
                self.sendValue(ERR_RETRY)

    # Actual handling functions.

    def handleInit(self,name,open_flags,dict_flags):
        self.name = name
        self.open_flags = open_flags
        self.dict_flags = dict_flags
        print "Name: %r" % name
        print "Open Flags:", open_flags
        print "Dict Flags:", dict_flags
        return dict_flags

    def handleLookup(self,key):
        try:
            print "Lookup for: %r" % key
            data = getattr(self,"%sLookup" % (self.name,))(key)
            print "Data: %r" % data
            return data
        except KeyError:
            print "No data."
            return None
        except Exception, e:
            print "Another error: %r, %s" % (e,e)
            raise
